@@ -71,48 +71,83 @@ class Experiment(ABC):
                 raise ValueError(f"Field {field.getName_field()} has an invalid shape: {field.field.shape}. Expected shape to be at least ({max_sample},).")
             self.AcousticFields[i].field = field.field[min_sample:max_sample, :, :]
 
+    def addNoise(self, noiseType='gaussian', noiseLvl=0.1, withTumor=True):
+        """
+        Ajoute du bruit (gaussien ou poisson) au signal AO sélectionné.
+
+        Args:
+            noiseType (str): Type de bruit à ajouter ('gaussian' ou 'poisson').
+            noiseLvl (float): Niveau de bruit (écart-type pour le bruit gaussien, facteur multiplicatif pour le bruit de Poisson).
+            withTumor (bool): Si True, ajoute le bruit au signal avec tumeur, sinon au signal sans tumeur.
+        """
+        if withTumor and self.AOsignal_withTumor is None:
+            raise ValueError("AO signal with tumor is not generated. Please generate it first.")
+        if not withTumor and self.AOsignal_withoutTumor is None:
+            raise ValueError("AO signal without tumor is not generated. Please generate it first.")
+
+        if withTumor:
+            AOsignal = self.AOsignal_withTumor
+        else:
+            AOsignal = self.AOsignal_withoutTumor
+
+        if noiseType.lower() == 'gaussian':
+            noise = np.random.normal(0, noiseLvl, AOsignal.shape)
+            noisy_signal = AOsignal + noise
+        elif noiseType.lower() == 'poisson':
+            # Pour le bruit de Poisson, on utilise souvent un facteur multiplicatif
+            # car le bruit de Poisson est proportionnel à la racine carrée du signal.
+            # Ici, on multiplie le signal par un facteur aléatoire centré autour de 1.
+            noise = np.random.poisson(noiseLvl * np.abs(AOsignal)) / (noiseLvl * np.abs(AOsignal).max())
+            noisy_signal = AOsignal * noise
+        else:
+            raise ValueError("noiseType must be either 'gaussian' or 'poisson'.")
+
+        if withTumor:
+            self.AOsignal_withTumor = noisy_signal
+        else:
+            self.AOsignal_withoutTumor = noisy_signal
+
+
     def reduceDims(self, mode='avg'):
         """
-        Réduit les dimensions T, Z, X d'une matrice (T, Z, X, N) ou (T, Z, X) par 2 en utilisant une convolution.
-        Args:
-            mode (str): 'avg' (moyenne) ou 'max' (max-pooling).
+        Réduit les dimensions T, X, Z d'un numpy array (T, X, Z) par 2 en utilisant une convolution.
+        Retourne un numpy array et met à jour les paramètres numériques.
         """
         for i in trange(len(self.AcousticFields),
-                        desc="Downsampling Acoustic Fields (T, Z, X → T//2, Z//2, X//2)"):
-            # Conversion en tenseur PyTorch si nécessaire
-            if isinstance(self.AcousticFields[i].field, np.ndarray):
-                self.AcousticFields[i].field = torch.from_numpy(self.AcousticFields[i].field)
+                        desc="Downsampling Acoustic Fields (T, X, Z → T//2, X//2, Z//2)"):
+            # Conversion en tenseur PyTorch
+            field = self.AcousticFields[i].field
+            if not isinstance(field, torch.Tensor):
+                field = torch.from_numpy(field)
 
-            # Vérification de la forme du tenseur
-            if self.AcousticFields[i].field.dim() == 3:
-                # Cas (T, Z, X) : ajout d'une dimension fictive N=1
-                x = self.AcousticFields[i].field.unsqueeze(-1)  # (T, Z, X, 1)
-            elif self.AcousticFields[i].field.dim() == 4:
-                # Cas (T, Z, X, N) : permutation normale
-                x = self.AcousticFields[i].field
-            else:
-                raise ValueError(f"Forme non supportée : {self.AcousticFields[i].field.shape}")
+            # Vérification de la forme (doit être 3D : T, X, Z)
+            if field.dim() != 3:
+                raise ValueError(f"Forme non supportée : {field.shape}. Attendu (T, X, Z).")
 
-            # Permutation des axes pour adapter à conv3d: (N, 1, T, Z, X)
-            x = x.permute(3, 0, 1, 2).unsqueeze(1)  # (N, 1, T, Z, X)
+            # Ajout des dimensions pour conv3d : (1, 1, T, X, Z)
+            x = field.unsqueeze(0).unsqueeze(0)
 
             # Réduction par convolution 3D
             if mode == 'avg':
                 x_down = F.avg_pool3d(x, kernel_size=(2, 2, 2), stride=(2, 2, 2))
-            elif mode == 'max':
+            else:  # mode == 'max'
                 x_down = F.max_pool3d(x, kernel_size=(2, 2, 2), stride=(2, 2, 2))
-            else:
-                raise ValueError("Mode invalide. Utilisez 'avg' ou 'max'.")
 
-            # Retour à l'ordre original (T//2, Z//2, X//2, N)
-            self.AcousticFields[i].field = x_down.squeeze(1).permute(1, 2, 3, 0)
+            # Conversion en numpy array et suppression des dimensions ajoutées
+            self.AcousticFields[i].field = x_down.squeeze(0).squeeze(0).cpu().numpy()
+
+        # Fonction utilitaire pour convertir et mettre à jour un paramètre
+        def convert_and_update(param_dict, key, operation):
+            if key in param_dict:
+                if isinstance(param_dict[key], str):
+                    param_dict[key] = float(param_dict[key])
+                param_dict[key] = operation(param_dict[key])
 
         # Mise à jour des paramètres
-        self.params.acoustic['f_saving'] /= 2
-        self.params.general['dx'] *= 2
-        self.params.general['dz'] *= 2
-        self.params.general['dy'] *= 2
-        
+        convert_and_update(self.params.acoustic, 'f_saving', lambda x: x / 2)
+        for param in ['dx', 'dy', 'dz']:
+            convert_and_update(self.params.general, param, lambda x: x * 2)
+
     def saveAcousticFields(self, save_directory):
         progress_bar = trange(len(self.AcousticFields), desc="Saving Acoustic Fields")
         for i in progress_bar:
@@ -231,7 +266,7 @@ class Experiment(ABC):
             self.AOsignal_withoutTumor = AOsignal
 
     @staticmethod
-    def _load_AOSignal(cdh_file):
+    def _loadAOSignal(cdh_file):
         with open(cdh_file, "r") as file:
             cdh_content = file.readlines()
 
@@ -299,7 +334,7 @@ class Experiment(ABC):
 
         print(f"Fichiers .cdf, .cdh et info.txt sauvegardés dans {save_directory}")
 
-    def show_AOsignal(self, withTumor=True, save_dir=None, wave_name=None):
+    def showAOsignal(self, withTumor=True, save_dir=None, wave_name=None):
         if withTumor and self.AOsignal_withTumor is None:
             raise ValueError("AO signal with tumor is not generated. Please generate it first.")
         if not withTumor and self.AOsignal_withoutTumor is None:
@@ -435,6 +470,21 @@ class Experiment(ABC):
         plt.close(fig)
         return ani
 
+    def showPhantom(self):
+        """
+        Displays the optical phantom with absorbers.
+        """
+        try:
+            plt.imshow(self.OpticImage.phantom, extent=(self.OpticImage.laser.x[0], self.OpticImage.laser.x[-1] + 1, self.OpticImage.laser.z[-1], self.OpticImage.laser.z[0]), aspect='equal', cmap='hot')
+            plt.colorbar(label='Intensity')
+            plt.xlabel('X (mm)', fontsize=20)
+            plt.ylabel('Z (mm)', fontsize=20)
+            plt.tick_params(axis='both', which='major', labelsize=20)
+            plt.title('Optical Phantom with Absorbers')
+            plt.show()
+        except Exception as e:
+            raise RuntimeError(f"Error plotting phantom: {e}")
+        
     @abstractmethod
     def check(self):
         """
