@@ -1,6 +1,6 @@
 from ._mainRecon import Recon
 from .ReconEnums import ReconType, OptimizerType, ProcessType
-from .AOT_Optimizers import MLEM
+from .AOT_Optimizers import MLEM, LS
 from .ReconTools import check_gpu_memory, calculate_memory_requirement, mse
 from AOT_biomaps.Config import config
 
@@ -23,7 +23,7 @@ class AlgebraicRecon(Recon):
     This class implements the Algebraic reconstruction process.
     It currently does not perform any operations but serves as a template for future implementations.
     """
-    def __init__(self, opti = OptimizerType.MLEM, numIterations = 10000, numSubsets = 1, isSavingEachIteration=True, **kwargs):
+    def __init__(self, opti = OptimizerType.MLEM, numIterations = 10000, numSubsets = 1, isSavingEachIteration=True, lambda_reg=None, L_Factor=None, **kwargs):
         super().__init__(**kwargs)
         self.reconType = ReconType.Algebraic
         self.optimizer = opti
@@ -32,6 +32,8 @@ class AlgebraicRecon(Recon):
         self.numIterations = numIterations
         self.numSubsets = numSubsets
         self.isSavingEachIteration = isSavingEachIteration
+        self.lambda_reg = lambda_reg
+        self.L_Factor = L_Factor
 
         if self.numIterations <= 0:
             raise ValueError("Number of iterations must be greater than 0.")
@@ -107,66 +109,93 @@ class AlgebraicRecon(Recon):
         plt.grid(True)
         plt.tight_layout()
 
-        if isSaving:
+        if isSaving and self.saveDir is not None:
             now = datetime.now()    
             date_str = now.strftime("%Y_%d_%m_%y")
-            SavingFolder = os.path.join(self.saveDir, 'results', f'MSE_plot{date_str}.png')
+            if self.optimizer == OptimizerType.MLEM:
+                SavingFolder = os.path.join(self.saveDir, 'results', f'MSE_plot_MLEM{date_str}.png')
+            elif self.optimizer == OptimizerType.LS:
+                SavingFolder = os.path.join(self.saveDir, 'results', f'MSE_plot_LS{date_str}.png')
+            elif self.optimizer == OptimizerType.LS_TV:
+                SavingFolder = os.path.join(self.saveDir, 'results', f'MSE_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}_{date_str}.png')
             plt.savefig(SavingFolder, dpi=300)
             print(f"MSE plot saved to {SavingFolder}")
         
         plt.show()
 
     def show_MSE_bestRecon(self, isSaving=True):
-        
         if not self.MSE:
             raise ValueError("MSE is empty. Please calculate MSE first.")
 
         best_idx = np.argmin(self.MSE)
         best_recon = self.reconPhantom[best_idx]
 
-        # ----------------- Plotting -----------------
-        fig, axs = plt.subplots(1, 3, figsize=(15, 5))  # 1 row, 3 columns
-
-        # Normalization based on LAMBDA max
-        lambda_max = np.max(self.experiment.OpticImage.laser.intensity)
+        # Crée la figure et les axes
+        fig, axs = plt.subplots(1, 3, figsize=(15, 5))
 
         # Left: Best reconstructed image (normalized)
-        im0 = axs[0].imshow(best_recon / lambda_max, 
-                            extent=(self.experiment.params.general['Xrange'][0], self.experiment.params.general['Xrange'][1], self.experiment.params.general['Zrange'][1], self.experiment.params.general['Zrange'][0]),
+        im0 = axs[0].imshow(best_recon / np.max(best_recon),
+                            extent=(self.experiment.params.general['Xrange'][0]*1000, self.experiment.params.general['Xrange'][1]*1000,
+                                    self.experiment.params.general['Zrange'][1]*1000, self.experiment.params.general['Zrange'][0]*1000),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
         axs[0].set_title(f"Min MSE Reconstruction\nIter {best_idx+1}, MSE={np.min(self.MSE):.4f}")
-        axs[0].set_xlabel("x (mm)")
-        axs[0].set_ylabel("z (mm)")
-        plt.colorbar(im0, ax=axs[0])
+        axs[0].set_xlabel("x (mm)", fontsize=12)
+        axs[0].set_ylabel("z (mm)", fontsize=12)
+        axs[0].tick_params(axis='both', which='major', labelsize=8)
 
         # Middle: Ground truth (normalized)
-        im1 = axs[1].imshow(self.experiment.OpticImage.laser.intensity / lambda_max, 
-                            extent=(self.experiment.params.general['Xrange'][0], self.experiment.params.general['Xrange'][1], self.experiment.params.general['Zrange'][1], self.experiment.params.general['Zrange'][0]),
+        im1 = axs[1].imshow(self.experiment.OpticImage.phantom / np.max(self.experiment.OpticImage.phantom),
+                            extent=(self.experiment.params.general['Xrange'][0]*1000, self.experiment.params.general['Xrange'][1]*1000,
+                                    self.experiment.params.general['Zrange'][1]*1000, self.experiment.params.general['Zrange'][0]*1000),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
         axs[1].set_title(r"Ground Truth ($\lambda$)")
-        axs[1].set_xlabel("x (mm)")
-        axs[1].set_ylabel("z (mm)")
-        plt.colorbar(im1, ax=axs[1])
+        axs[1].set_xlabel("x (mm)", fontsize=12)
+        axs[1].set_ylabel("z (mm)", fontsize=12)
+        axs[1].tick_params(axis='both', which='major', labelsize=8)
+        axs[1].tick_params(axis='y', which='both', left=False, right=False, labelleft=False)
 
-        # Right: Reconstruction at iter 350
-        lastRecon = self.reconPhantom[-1] 
-        im2 = axs[2].imshow(lastRecon / lambda_max,
-                            extent=(self.experiment.params.general['Xrange'][0], self.experiment.params.general['Xrange'][1], self.experiment.params.general['Zrange'][1], self.experiment.params.general['Zrange'][0]),
+        # Right: Reconstruction at last iteration
+        lastRecon = self.reconPhantom[-1]
+        if self.experiment.OpticImage.phantom.shape != lastRecon.shape:
+            lastRecon = lastRecon.T
+        im2 = axs[2].imshow(lastRecon / np.max(lastRecon),
+                            extent=(self.experiment.params.general['Xrange'][0]*1000, self.experiment.params.general['Xrange'][1]*1000,
+                                    self.experiment.params.general['Zrange'][1]*1000, self.experiment.params.general['Zrange'][0]*1000),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
-        axs[2].set_title(f"Last Reconstruction\nIter {self.numIterations * self.numSubsets}, MSE={np.mean((self.experiment.OpticImage.laser.intensity - lastRecon) ** 2):.4f}")
-        axs[2].set_xlabel("x (mm)")
-        axs[2].set_ylabel("z (mm)")
-        plt.colorbar(im2, ax=axs[2])
+        axs[2].set_title(f"Last Reconstruction\nIter {self.numIterations * self.numSubsets}, MSE={np.mean((self.experiment.OpticImage.phantom - lastRecon) ** 2):.4f}")
+        axs[2].set_xlabel("x (mm)", fontsize=12)
+        axs[2].set_ylabel("z (mm)", fontsize=12)
+        axs[2].tick_params(axis='both', which='major', labelsize=8)
 
-        plt.tight_layout()
-        if isSaving:
-            now = datetime.now()    
+        # Ajoute une colorbar horizontale centrée en dessous des trois plots
+        fig.subplots_adjust(bottom=0.2)
+        cbar_ax = fig.add_axes([0.25, 0.08, 0.5, 0.03])
+        cbar = fig.colorbar(im2, cax=cbar_ax, orientation='horizontal')
+        cbar.set_label('Normalized Intensity', fontsize=12)
+        cbar.ax.tick_params(labelsize=8)
+
+        plt.subplots_adjust(wspace=0.3)
+
+        if isSaving and self.saveDir is not None:
+            now = datetime.now()
             date_str = now.strftime("%Y_%d_%m_%y")
-            SavingFolder = os.path.join(self.saveDir, 'results', f'comparison_MSE_BestANDLastRecon{date_str}.png')
-            plt.savefig(SavingFolder, dpi=300)
+            savePath = os.path.join(self.saveDir, 'results')
+            if not os.path.exists(savePath):
+                os.makedirs(savePath)
+            if self.optimizer == OptimizerType.MLEM:
+                namePath = f'{self.SMatrix.shape[3]}_SCANS_comparison_MSE_BestANDLastRecon_MLEM_Date_{date_str}.png'
+            elif self.optimizer == OptimizerType.LS:
+                namePath = f'{self.SMatrix.shape[3]}_SCANS_comparison_MSE_BestANDLastRecon_LS_Date_{date_str}.png'
+            elif self.optimizer == OptimizerType.LS_TV:
+                namePath = f'{self.SMatrix.shape[3]}_SCANS_comparison_MSE_BestANDLastRecon_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}_Date_{date_str}.png'
+            SavingFolder = os.path.join(savePath, namePath)
+            plt.savefig(SavingFolder, dpi=300, bbox_inches='tight')
             print(f"MSE plot saved to {SavingFolder}")
+
         plt.show()
 
+
+
     def show_theta_animation(self, vmin=None, vmax=None, duration=5000, save_path=None):
         """
         Show theta iteration animation (for Jupyter) and optionally save it as a GIF.
@@ -518,15 +547,23 @@ class AlgebraicRecon(Recon):
                 raise ValueError("AO signal with tumor is not available. Please generate AO signal with tumor the experiment first in the experiment object.")
             if self.optimizer.value == OptimizerType.MLEM.value:
                 self.reconPhantom = self._MLEM(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
+            elif self.optimizer.value == OptimizerType.LS.value:
+                self.reconPhantom = self._LS(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
+            elif self.optimizer.value == OptimizerType.LS_TV.value:
+                self.reconPhantom = self._LS_Regularized(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
             else:
-                raise ValueError(f"Only MLEM is supported for simple algebraic reconstruction. {self.optimizer.value} need Bayesian reconstruction")
+                raise ValueError(f"Only MLEM and LS are supported for simple algebraic reconstruction. {self.optimizer.value} need Bayesian reconstruction")
         else:
             if self.experiment.AOsignal_withoutTumor is None:
                 raise ValueError("AO signal without tumor is not available. Please generate AO signal without tumor the experiment first in the experiment object.")
             if self.optimizer.value == OptimizerType.MLEM.value:
                 self.reconLaser = self._MLEM(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
+            elif self.optimizer.value == OptimizerType.LS.value:
+                self.reconLaser = self._LS(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
+            elif self.optimizer.value == OptimizerType.LS_TV.value:
+                self.reconLaser = self._LS_Regularized(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
             else:
-                raise ValueError(f"Only MLEM is supported for simple algebraic reconstruction. {self.optimizer.value} need Bayesian reconstruction")
+                raise ValueError(f"Only MLEM and LS are supported for simple algebraic reconstruction. {self.optimizer.value} need Bayesian reconstruction")
 
     def _AlgebraicReconCASToR(self, withTumor):
         
@@ -626,4 +663,39 @@ class AlgebraicRecon(Recon):
                 warnings.warn("Optimized MLEM failed. Falling back to basic CPU MLEM.")
                 result = MLEM._MLEM_CPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
 
-        return result
\ No newline at end of file
+        return result
+    
+    def _LS(self, SMatrix, y, withTumor):
+        """
+        This method implements the LS algorithm using either CPU or single-GPU PyTorch acceleration.
+        Multi-GPU mode is disabled due to memory fragmentation issues or lack of availability.
+        """
+        result = None
+        required_memory = calculate_memory_requirement(SMatrix, y)
+
+        if self.isGPU:
+            if check_gpu_memory(config.select_best_gpu(), required_memory):
+                result = LS._LS_GPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+            else:
+                warnings.warn("Insufficient GPU memory for single GPU LS. Falling back to CPU.")
+
+        if result is None and self.isMultiCPU:
+            result = LS._LS_CPU_multi(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+
+        if result is None:
+            result = LS._LS_CPU_opti(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+            if result is None:
+                warnings.warn("Optimized LS failed. Falling back to basic CPU LS.")
+                result = LS._LS_CPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
+
+        return result
+
+    def _LS_Regularized(self,SMatrix, y, withTumor):
+        required_memory = calculate_memory_requirement(SMatrix, y)
+        if check_gpu_memory(config.select_best_gpu(), required_memory):
+            if self.lambda_reg is None or self.L_Factor is None:
+                raise ValueError("For LS with TV regularization, both lambda_reg and L_Factor must be specified.")
+            return LS._LS_TV_GPU(SMatrix= SMatrix, y=y,  numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor, lambda_tv=self.lambda_reg, L_Factor=self.L_Factor)
+        else:
+            warnings.warn("Insufficient GPU memory for single GPU LS with TV regularization. Falling back to CPU.")
+            raise NotImplementedError("LS with TV regularization is not implemented for CPU.")
\ No newline at end of file
